require 'logger'

# Manages a logger that can be used throughout the pubsub gem
# The logger can be set to any object that quacks like a logger including the
# Rails logger if so desired
#
# The logger can be set in the initializer of a program or anywhere throughout as
# HonestPubsub.logger = Rails.logger for example

module HonestPubsub
  module Logging
    # Sets the logger
    #
    # @param logger The logger to use throughout the pubsub gem
    # @return The logger we use throughout the gem
    def self.logger=(logger)
      raise StandardError("Can't set logger to nil") unless logger.present?
      @logger = logger
    end

    # Gets the logger
    # If no logger is defined when this method is called it will return a standard
    # ruby logger
    #
    # @return The logger we use throughout the gem
    def self.logger 
      config = HonestPubsub::Configuration.configuration[:logger]     
      @logger ||= create_logger( {}.
        merge( config[:level].present? ? { log_level: ::Logger::Severity.const_get(config[:level].upcase) } : {} ).
        merge( config[:file].present? ? { log_target: config[:file] } : {} ) )
    end

    # Builds a standard ruby Logger
    #
    # @return Returns the logger at the end of the method
    def self.create_logger( options = {} )
      @logger = ::Logger.new(options.fetch(:log_target){ STDOUT })
      @logger.level = options.fetch(:log_level){ ::Logger::INFO }
      @logger
    end
  end
end