require 'rubygems'
require 'active_support/core_ext'

# This class provders the worker for executing the subscriber. It receives a message from rabbitmq and
# creates an instance of the subscriber to execute with the message and context

module HonestPubsub
  class Subscriber
    @@registered_subscribers = []
    class_attribute :payload_validators, :error_handlers, :subscribed_key, :subscribed_queue
    attr_accessor :delivery_routing_data, :delivery_properties, :context

    def self.inherited(klass)
      @@registered_subscribers << klass
    end

    # Specify the routing key that the subscriber class should listen to.
    # @param [String] routing_key_name The routing key to subscribe to. Must be characters only separated by periods (.)
    # @param [Hash] options Allowed option is :on to optionally specify a queue. If not provided, queue name is generated from the routing key
    def self.subscribe_to(routing_key_name, options = {})
      options.assert_valid_keys(:on)
      unless validate_routing_key_name(routing_key_name)
        raise ArgumentError.new("#{routing_key_name} is not supported. Only lower case characters separated by periods are allowed.")
      end
      self.subscribed_key   = routing_key_name
      self.subscribed_queue = generated_queue_name(routing_key_name, options[:on])
    end

    # Sets the validator for payload
    #
    # @param validator The validator to use for validating the payload.
    #                  Returns false if the payload is not valid.
    #                  Proc must accept a payload as an argument.
    def self.validates_payload_with(*validators)
      self.payload_validators ||= []
      self.payload_validators += validators
    end

    # Sets an error handler for the class
    def self.handle_errors_with(handler)
      error_handler = handler
    end

    # @param [Hash] context Context from invocation
    # @param [Object] delivery_routing_data Contains routing information like originator and routing key
    # @param [Object] delivery_properties
    def initialize(delivery_routing_data, delivery_properties, context)
      @delivery_routing_data = delivery_routing_data
      @delivery_properties   = delivery_properties
      @context               = context
    end

    # Performs validation if validates_payload_with is defined and then calls the perform method
    # @param [Object] payload Payload of the message
    def perform!(payload)
      if !valid_payload?(payload)
        HonestPubsub.logger.error("Payload validation failed for #{self.class.name}")
        raise ::HonestPubsub::PayloadValidationError.new("Invalid Payload for #{self.class.name}")
      end

      perform(context, payload)
    end

    # Actual subscribers need to implement perform method. This is the method where the message is actually processed.
    # @param [Object] payload Payload of the message
    def perform(payload)
      raise "Need implementation for your worker."
    end

    # @return [String] The original routing key with which the current message was published
    def routing_key
      delivery_routing_data[:routing_key]
    end

    # Iterates over all the payload validators and returns false if any of them are false
    # @param [Object] payload The payload/arguments of the message
    # @return [Boolen] Should return true or false value - If no validators are specified, then returns true
    def valid_payload?(payload)
      return true unless payload_validators.present?

      payload_validators.inject(true) { |is_valid, validator|
        is_valid && (validator.respond_to?(:call) ? validator.call(payload) : send(validator, payload))
      }
    end

    private

    def self.validate_routing_key_name(key)
      return true if key.blank?
      key.match(/\A([a-z]+\.?)*([a-z]+)\Z/).present?
    end

    def self.generated_queue_name(routing_key, queue_name)
      return queue_name if queue_name.present?
      [HonestPubsub::Configuration.application_name.to_s.gsub(/[^\w\_]/, ''), routing_key.gsub(".", '_')].reject(&:blank?).join('_')
    end

  end
end